/*
 * Sumit Khanna - PenguinDreams.org
 *   Free for educational and non-commercial use
 */
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintStream;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.SecureRandom;
import java.security.Security;
import java.security.Signature;

import javax.xml.bind.DatatypeConverter;

import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.jce.provider.JDKKeyPairGenerator;
import org.bouncycastle.openssl.PEMReader;
import org.bouncycastle.openssl.PEMWriter;

class JavaSign {

  public static PrintStream out = System.out;
  public static PrintStream err = System.err;

  public static void main(String[] args) throws Exception {
	  
    if(args.length != 2) {
      err.println("Usage: java JavaSign <pem file> <data file to sign>");
      System.exit(1);
    }
    
    File pemFile = new File(args[0]);
    File dataFile = new File(args[1]);
    
    if(!dataFile.exists()) {
    	err.println("Data File Does Not Exist");
    	System.exit(1);
    }
    
    Security.addProvider(new BouncyCastleProvider());
    KeyPair keys = null;
    
    if(!pemFile.exists()) {
    	
    	err.println("PEM File Does Not Exist. Generating.");
    	KeyPairGenerator r = KeyPairGenerator.getInstance("RSA");
    	
    	//keysize in bits is 2048
    	r.initialize(2048,new SecureRandom());
    	keys = r.generateKeyPair();
    	PEMWriter pemWriter = new PEMWriter(new FileWriter(pemFile));
    	pemWriter.writeObject(keys);
    	pemWriter.close(); //You must flush or close the file or else it will not save
    }
    else {
    	keys = (KeyPair) new PEMReader(new FileReader(pemFile)).readObject();
    }
    
    //read data file into signature instance
    FileInputStream fin = new FileInputStream(dataFile);
    byte[] data = new byte[(int) dataFile.length()];
    fin.read(data);
    
    //Sign the data
    Signature sg = Signature.getInstance("SHA1withRSA");
    sg.initSign(keys.getPrivate());
    sg.update(data);
    
    //output base64 encoded binary signature 
    out.println(DatatypeConverter.printBase64Binary(sg.sign()));

  }


}
